@@ -1,6 +1,6 @@
 /* bug-buddy bug submitting program
  *
- * Copyright (C) The Free Software Foundation
+ * Copyright (C) Jacob Berkman
  *
  * Author:  Jacob Berkman  <jberkman@andrew.cmu.edu>
  *
@@ -24,29 +24,35 @@
 #include <gnome.h>
 #include <glade/glade.h>
 
+#include <signal.h>
+
 #include "bug-buddy.h"
 
 /* libglade callbacks */
-gboolean on_nature_page_next (GtkWidget *, GtkWidget *);
-gboolean on_attach_page_next (GtkWidget *, GtkWidget *);
-gboolean on_core_page_back (GtkWidget *, GtkWidget *);
+gboolean on_nature_page_next  (GtkWidget *, GtkWidget *);
+gboolean on_attach_page_next  (GtkWidget *, GtkWidget *);
+gboolean on_core_page_back    (GtkWidget *, GtkWidget *);
 gboolean on_less_page_prepare (GtkWidget *, GtkWidget *);
-gboolean on_less_page_back (GtkWidget *, GtkWidget *);
-gboolean on_misc_page_back (GtkWidget *, GtkWidget *);
-gboolean on_the_druid_cancel (GtkWidget *);
+gboolean on_less_page_back    (GtkWidget *, GtkWidget *);
+gboolean on_less_page_next    (GtkWidget *, GtkWidget *);
+gboolean on_misc_page_back    (GtkWidget *, GtkWidget *);
+gboolean on_the_druid_cancel  (GtkWidget *);
 gboolean on_complete_page_prepare (GtkWidget *, GtkWidget *);
-gboolean on_complete_page_finish (GtkWidget *, GtkWidget *);
-gboolean on_system_page_prepare (GtkWidget *, GtkWidget *);
-gboolean on_gnome_page_prepare (GtkWidget *, GtkWidget *);
-gboolean on_contact_page_next (GtkWidget *, GtkWidget *);
-void on_version_list_select_row (GtkCList *list, gint row, gint col,
-				 GdkEventButton *event, gpointer udata);
-void on_version_edit_activate (GtkEditable *editable, gpointer user_data);
-void on_version_apply_clicked (GtkButton *button, gpointer udata);
-void on_file_radio_toggled (GtkWidget *radio, gpointer data);
-gboolean on_action_page_next (GtkWidget *page, GtkWidget *druid);
+gboolean on_complete_page_finish  (GtkWidget *, GtkWidget *);
+gboolean on_system_page_prepare   (GtkWidget *, GtkWidget *);
+gboolean on_gnome_page_prepare    (GtkWidget *, GtkWidget *);
+gboolean on_contact_page_next     (GtkWidget *, GtkWidget *);
+void on_version_list_select_row   (GtkCList *list, gint row, gint col,
+				   GdkEventButton *event, gpointer udata);
+void update_selected_row      (GtkWidget *widget, gpointer data);
+void on_version_apply_clicked (GtkButton *button, gpointer data);
+void on_file_radio_toggled    (GtkWidget *radio, gpointer data);
+gboolean on_action_page_next  (GtkWidget *page, GtkWidget *druid);
+gboolean on_action_page_back  (GtkWidget *page, GtkWidget *druid);
 GtkWidget * make_anim (gchar *widget_name, gchar *string1, 
 		       gchar *string2, gint int1, gint int2);
+GtkWidget * make_pixmap_button (gchar *widget_name, gchar *string1, 
+				gchar *string2, gint int1, gint int2);
 
 extern const char *packages[];
 
@@ -58,10 +64,10 @@ const gchar *severity[] = {
 	NULL };
 
 const gchar *bug_class[][2] = {
-	{ N_("software bug"), "sw-bug" },
-	{ N_("documentaion bug"), "doc-bug" },
-	{ N_("change request"), "change-request" },
-	{ N_("support"), "support" },
+	{ N_("software bug"),      "sw-bug" },
+	{ N_("documentation bug"), "doc-bug" },
+	{ N_("change request"),    "change-request" },
+	{ N_("support"),           "support" },
 	{ NULL, NULL } };
 
 struct {
@@ -84,18 +90,14 @@ struct {
 DruidData druid_data;
 
 static ListData list_data[] = {
-	{ N_("Operating System"), { "uname -a" } },
+	{ N_("System"), { "uname -a" } },
 	{ N_("Distribution"),
-	  { "( [ -f /etc/slackware-version ] && echo \"Slackware\") || "
-	    "( [ -f /etc/debian_version ] && echo \"Debian\") || "
-	    "( [ -f /etc/redhat-release ] && echo \"Red Hat\") || "
-	    "( [ -f /etc/SuSE-release ]   && echo \"SuSE\") || "
-	    "echo \"\"" } },
-	{ N_("Distribution version"),
-	  { "( [ -f /etc/slackware-version ] && cat /etc/slackware-version) || "
-	    "( [ -f /etc/debian_version ] && cat /etc/debian_version) || "
+	  { "( [ -f /etc/slackware-version ] && "
+	    "  echo -n \"Slackware \" && cat /etc/slackware-version) || "
+	    "( [ -f /etc/slackware-version ] && "
+	    "  echo -n \"Debian \" && cat /etc/debian_version) || "
 	    "( [ -f /etc/redhat-release ] && cat /etc/redhat-release) || "
-	    "( [ -f /etc/SuSE-release ]   && head -1 /etc/SuSE-release) || "
+	    "( [ -f /etc/SuSE-release ]   && head -1 /etc/SuSE-release) ||"
 	    "echo \"\"" } },
 	{ N_("C library"), { "rpm -q glibc",  "rpm -q libc" } },
 	{ N_("C Compiler"), { "gcc --version", "cc -V" } },
@@ -129,33 +131,29 @@ static const struct poptOption options[] = {
 };
 
 static void
-save_config ()
+save_entry (const char *name, const char *name2, const char *save_path)
 {
 	GtkWidget *w;
 	gchar *s;
 
-	w = glade_xml_get_widget (druid_data.xml, "name_entry");
+	w = glade_xml_get_widget (druid_data.xml, name);
 	s = gtk_entry_get_text (GTK_ENTRY (w));
-	gnome_config_set_string ("/bug-buddy/last/name", s);
-	w = glade_xml_get_widget (druid_data.xml, "name_entry2");
+	gnome_config_set_string (save_path, s);
+	w = glade_xml_get_widget (druid_data.xml, name2);
+	if (GNOME_IS_FILE_ENTRY (w))
+		w = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (w));
 	gnome_entry_prepend_history (GNOME_ENTRY (w), TRUE, s);
 	gnome_entry_save_history (GNOME_ENTRY (w));
+}
 
-	w = glade_xml_get_widget (druid_data.xml, "email_entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-	gnome_config_set_string ("/bug-buddy/last/email", s);
-	w = glade_xml_get_widget (druid_data.xml, "email_entry2");
-	gnome_entry_prepend_history (GNOME_ENTRY (w), TRUE, s);
-	gnome_entry_save_history (GNOME_ENTRY (w));
-
-	w = glade_xml_get_widget (druid_data.xml, "file_entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-	gnome_config_set_string ("/bug-buddy/last/bugfile", s);
-
-	w = glade_xml_get_widget (druid_data.xml, "file_entry2");
-	w = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (w));
-	gnome_entry_save_history (GNOME_ENTRY (w));
+static void
+save_config ()
+{
 
+	save_entry ("name_entry",  "name_entry2",  "/bug-buddy/last/name");
+	save_entry ("email_entry", "email_entry2", "/bug-buddy/last/email");
+	save_entry ("file_entry",  "file_entry2",  "/bug-buddy/last/bugfile");
+	
 	gnome_config_set_int ("/bug-buddy/last/submittype", 
 			      druid_data.submit_type);
 
@@ -163,40 +161,36 @@ save_config ()
 }
 
 static void
-load_config ()
+load_entry (const char *name, const char *name2,
+	    const char *path, const char *def)
 {
 	GtkWidget *w;
 	gchar *s;
 
-	w = glade_xml_get_widget (druid_data.xml, "name_entry");
-	s = gnome_config_get_string ("/bug-buddy/last/name");
+	w = glade_xml_get_widget (druid_data.xml, name);
+	s = gnome_config_get_string (path);
 	if (s)
 		gtk_entry_set_text (GTK_ENTRY (w), s);
-	else
-		gtk_entry_set_text (GTK_ENTRY (w), g_get_real_name ());
+	else if (def)
+		gtk_entry_set_text (GTK_ENTRY (w), def);
 	g_free (s);
-	w = glade_xml_get_widget (druid_data.xml, "name_entry2");
+	w = glade_xml_get_widget (druid_data.xml, name2);
+	if (GNOME_IS_FILE_ENTRY (w))
+		w = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (w));
 	gnome_entry_load_history (GNOME_ENTRY (w));
+}
 
-	w = glade_xml_get_widget (druid_data.xml, "email_entry");
-	s = gnome_config_get_string ("/bug-buddy/last/email");
-	if (s)
-		gtk_entry_set_text (GTK_ENTRY (w), s);
-	else 
-		gtk_entry_set_text (GTK_ENTRY (w), g_get_user_name ());
-	g_free (s);
-	w = glade_xml_get_widget (druid_data.xml, "email_entry2");
-	gnome_entry_load_history (GNOME_ENTRY (w));
+static void
+load_config ()
+{
+	load_entry ("name_entry", "name_entry2",
+		    "/bug-buddy/last/name", g_get_real_name ());
 
-	w = glade_xml_get_widget (druid_data.xml, "file_entry");
-	s = gnome_config_get_string ("/bug-buddy/last/bugfile");
-	if (s)
-		gtk_entry_set_text (GTK_ENTRY (w), s);
-	g_free (s);
+	load_entry ("email_entry", "email_entry2",
+		    "/bug-buddy/last/email", g_get_user_name ());
 
-	w = glade_xml_get_widget (druid_data.xml, "file_entry2");
-	w = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (w));
-	gnome_entry_load_history (GNOME_ENTRY (w));
+	load_entry ("file_entry", "file_entry2",
+		    "/bug-buddy/last/bugfile", NULL);
 
 	druid_data.submit_type = 
 		gnome_config_get_int ("/bug-buddy/last/submittype=0");
@@ -237,7 +231,7 @@ on_nature_page_next (GtkWidget *page, GtkWidget *druid)
 		newpage = druid_data.core;
 		break;
 	default:
-		newpage = druid_data.misc;
+		newpage = druid_data.action;
 		break;
 	}
 	gnome_druid_set_page (GNOME_DRUID (druid),
@@ -261,50 +255,45 @@ on_core_page_back (GtkWidget *page, GtkWidget *druid)
 	return TRUE;
 }
 
+static void
+on_refresh_button_clicked (GtkWidget *w, gpointer data)
+{
+	druid_data.explicit_dirty = TRUE;
+	start_gdb ();
+}
+
 gboolean
 on_less_page_prepare (GtkWidget *page, GtkWidget *druid)
 {
-	static gchar *old_app = NULL;
-	static gchar *old_extra = NULL;
-	static CrashType old_type = -1;
-	
-	gchar *app = NULL, *extra = NULL;
-
-	g_message (_("obtaining stack trace..."));
-	
-	switch (druid_data.crash_type) {
-	case CRASH_DIALOG:
-		app = gtk_entry_get_text (GTK_ENTRY (druid_data.app_file));
-		extra = gtk_entry_get_text (GTK_ENTRY (druid_data.pid));
-		if ((old_type != CRASH_DIALOG) ||
-		    (!old_app || strcmp (app, old_app)) ||
-		    (!old_extra || strcmp (extra, old_extra))) {
-			get_trace_from_pair (app, extra);
-		}
-		break;
-	case CRASH_CORE:
-		extra = gtk_entry_get_text (GTK_ENTRY (druid_data.core_file));
-		if ((old_type != CRASH_CORE) ||
-		    (!old_extra || strcmp (extra, old_extra))) {
-			get_trace_from_core (extra);
-		}
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-
-	g_free (old_extra);
-	old_extra = g_strdup (extra);
-
-	g_free (old_app);
-	old_app = g_strdup (app);
-
-	old_type = druid_data.crash_type;
-
+	start_gdb ();
 	return FALSE;
 }
 
+static void
+on_stop_button_clicked (GtkWidget *button, gpointer data)
+{
+	GtkWidget *w;
+	if (!druid_data.fd)
+		return;
+
+	w = gnome_question_dialog (_("gdb has not finished getting the"
+				     "debugging information.\n"
+				     "Kill the gdb process (the stack trace"
+				     "will be incomplete)?"),
+				   NULL, NULL);
+
+	if (GNOME_YES == gnome_dialog_run_and_close (GNOME_DIALOG (w))) {
+		if (druid_data.gdb_pid == 0) {
+			g_warning (_("gdb has already exited"));
+			return;
+		}
+		kill (druid_data.gdb_pid, SIGTERM);
+		stop_gdb ();		
+		kill (druid_data.app_pid, SIGCONT);
+		druid_data.explicit_dirty = TRUE;
+	}
+}
+
 gboolean
 on_less_page_back (GtkWidget *page, GtkWidget *druid)
 {
@@ -326,7 +315,7 @@ on_less_page_back (GtkWidget *page, GtkWidget *druid)
 }
 
 gboolean
-on_misc_page_back (GtkWidget *page, GtkWidget *druid)
+on_action_page_back (GtkWidget *page, GtkWidget *druid)
 {
 	if (druid_data.crash_type != CRASH_NONE)
 		return FALSE;
@@ -449,7 +438,7 @@ on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 		/* fall through */
 	case SUBMIT_REPORT:
 		/*s3 = g_strconcat (druid_data.mail_cmd, s2, NULL);*/
-		g_message ("about to run '%s'", druid_data.mail_cmd);
+		g_message (_("about to run '%s'"), druid_data.mail_cmd);
 		fp = popen (druid_data.mail_cmd, "w");
 		if (!fp) {
 			s = g_strdup_printf (_("Unable to start mail program:\n"
@@ -534,7 +523,9 @@ on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 	g_free (s);
 
 	status = pclose (fp);
-	g_message ("Subprocess exited with status %d\n", status);
+#if 0
+	g_message (_("Subprocess exited with status %d"), status);
+#endif
 	save_config ();
 	gtk_main_quit ();
 
@@ -547,14 +538,17 @@ on_version_list_select_row (GtkCList *list, gint row, gint col,
 {	
 	gchar *s;
 	druid_data.selected_data = gtk_clist_get_row_data (list, row);
-	gtk_clist_get_text (list, row, 1, &s);
-	gtk_entry_set_text (GTK_ENTRY (druid_data.version_edit), s);
+	if (gtk_clist_get_text (list, row, 1, &s))
+		gtk_entry_set_text (GTK_ENTRY (druid_data.version_edit), s);
+	else
+		gtk_editable_delete_text (GTK_EDITABLE (druid_data.version_edit),
+					  0, -1);
 	gtk_clist_get_text (list, row, 0, &s);
 	gtk_label_set_text (GTK_LABEL (druid_data.version_label), _(s));
 }
 
-static void
-update_selected_row ()
+void
+update_selected_row (GtkWidget *w, gpointer data)
 {
 	gchar *s;
 	gint row;
@@ -562,18 +556,7 @@ update_selected_row ()
 		return;
 	row = druid_data.selected_data->row;
 	s = gtk_entry_get_text (GTK_ENTRY (druid_data.version_edit));
-	gtk_clist_set_text (GTK_CLIST (druid_data.version_list), row, 1, s);}
-
-void
-on_version_edit_activate (GtkEditable *editable, gpointer user_data)
-{
-	update_selected_row ();
-}
-
-void
-on_version_apply_clicked (GtkButton *button, gpointer udata)
-{
-	update_selected_row ();
+	gtk_clist_set_text (GTK_CLIST (druid_data.version_list), row, 1, s);
 }
 
 void
@@ -623,6 +606,7 @@ static gchar *
 get_data_from_command (const gchar *cmd)
 {
 	static gchar buf[1024];
+	int status;
 	FILE *fp;
 
 	fp = popen (cmd, "r");
@@ -637,7 +621,10 @@ get_data_from_command (const gchar *cmd)
 		return NULL;
 	} 
 
-	fclose (fp);
+	status = pclose (fp);
+#if 0
+	g_message (_("Subprocess exited with status %d"), status);
+#endif
 
 	return g_strchomp (buf);	
 }
@@ -657,39 +644,119 @@ set_bug_class (GtkWidget *w, gpointer data)
 }
 
 GtkWidget *
-make_anim (gchar *widget_name, gchar *string1, 
-	   gchar *string2, gint int1, gint int2)
+make_pixmap_button (gchar *widget_name, gchar *text, 
+		    gchar *s2, int stop, int i2)
+{
+	GtkWidget *p, *w;
+	char *type = stop 
+		? GNOME_STOCK_PIXMAP_STOP
+		: GNOME_STOCK_PIXMAP_REFRESH;
+
+	p = gnome_stock_pixmap_widget (NULL, type);
+	w = gnome_pixmap_button (p, text);
+	GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
+	gtk_container_set_border_width (GTK_CONTAINER (w), GNOME_PAD_SMALL);
+	return w;
+}
+
+GtkWidget *
+make_anim (gchar *widget_name, gchar *imgname, 
+	   gchar *string2, gint size, gint freq)
 {
 	gchar *pixmap;
 	druid_data.gdb_anim = gnome_animator_new_with_size (48, 48);
 	gnome_animator_set_loop_type (GNOME_ANIMATOR (druid_data.gdb_anim),
 				      GNOME_ANIMATOR_LOOP_RESTART);
-	pixmap = BUDDY_DATADIR "/bug-anim.png";
+	pixmap = g_strconcat (BUDDY_DATADIR, imgname, NULL);
 	gnome_animator_append_frames_from_file (GNOME_ANIMATOR (druid_data.gdb_anim),
-						pixmap, 0, 0, 250, 48);
+						pixmap, 0, 0, freq, size);
+	g_free (pixmap);
 	return druid_data.gdb_anim;
 }
 
+static gboolean
+clean_hash (gpointer key, gpointer value, gpointer data)
+{
+	g_free (key);
+	g_free (value);
+	return TRUE;
+}
+
+/* ugly stupid dumb stuff stolen from the bug reporting web page */
+static gchar *
+get_package_from_appname (const char *appname)
+{
+	FILE *fp;
+	gchar *package;
+	char buf[1024];
+	GHashTable *table;
+
+	fp = fopen (BUDDY_DATADIR "/prog.bugmap", "r");
+	if (!fp) {
+		g_warning ("Could not open bugfile '"
+			   BUDDY_DATADIR
+			   "/prog.bugmap' for reading.");
+		return NULL;
+	}
+
+	table = g_hash_table_new (g_str_hash, g_str_equal);
+	g_hash_table_freeze (table);
+
+	while (fgets (buf, 1024, fp)) {
+		package = strchr (buf, ' ');
+		if (!package)
+			continue;
+		*package = '\0';
+
+		g_hash_table_insert (table, 
+				     g_strdup (buf), 
+				     g_strdup (g_strstrip (package+1)));
+	}
+
+	fclose (fp);
+
+	g_hash_table_thaw (table);
+	package = g_strdup (g_hash_table_lookup (table,
+						 g_filename_pointer (appname)));
+	
+	g_hash_table_foreach_remove (table, clean_hash, NULL);
+	g_hash_table_destroy (table);
+
+	return package;
+}
+
+static void 
+init_toggle (const char *name, int test, int data, GtkSignalFunc func)
+{
+	GtkWidget *w;
+	w = glade_xml_get_widget (druid_data.xml, name);
+	if (test == data)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      TRUE);
+	gtk_signal_connect (GTK_OBJECT (w), "toggled", func,
+			    GINT_TO_POINTER (data));
+}
+
 static void
-init_ui (GladeXML *xml)
+init_ui ()
 {
 	GtkWidget *w, *m;
 	ListData *data;
 	gchar *row[3] = { NULL };
-	gchar *s;
+	gchar *s, *package = NULL;
 	int i;
 
-	glade_xml_signal_autoconnect(xml);
+	glade_xml_signal_autoconnect (druid_data.xml);
 
 	load_config ();
 
-	w = glade_xml_get_widget (xml, "the_druid");
+	w = glade_xml_get_widget (druid_data.xml, "the_druid");
 	druid_data.the_druid = w;
 
-	w = glade_xml_get_widget (xml, "package_entry2");
+	w = glade_xml_get_widget (druid_data.xml, "package_entry2");
 	for (i = 0; packages[i]; i++)
 		gnome_entry_prepend_history (GNOME_ENTRY (w), FALSE,
-					    packages[i]);
+					     packages[i]);
 
 	m = gtk_menu_new ();
 	for (i = 0; severity[i]; i++) {
@@ -700,7 +767,8 @@ init_ui (GladeXML *xml)
 		gtk_widget_show (w);
 		gtk_menu_append (GTK_MENU (m), w);
 	}
-	w = glade_xml_get_widget (xml, "severity_option");
+
+	w = glade_xml_get_widget (druid_data.xml, "severity_option");
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (w), 2);
 	druid_data.severity = 2;
@@ -714,112 +782,103 @@ init_ui (GladeXML *xml)
 		gtk_widget_show (w);
 		gtk_menu_append (GTK_MENU (m), w);
 	}
-	w = glade_xml_get_widget (xml, "class_option");
+
+	w = glade_xml_get_widget (druid_data.xml, "class_option");
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
 
 	/* dialog crash page */
-	w = glade_xml_get_widget (xml, "app_file");
-	if (popt_data.app_file)
-		s = popt_data.app_file;
-	else {
+	w = druid_data.app_file = 
+		glade_xml_get_widget (druid_data.xml, "app_file");
+	s = popt_data.app_file;
+	if (!s) {
 		s = getenv ("GNOME_CRASHED_APPNAME");
 		if (s)
-			g_warning ("$GNOME_CRASHED_APPNAME is deprectated,"
-				   "Please use the --appname command line"
-				   "argument instead.\n");
+			g_warning (_("$GNOME_CRASHED_APPNAME is deprecated.\n"
+				     "Please use the --appname command line"
+				     "argument instead."));
 	}
 
-	if (s) {
+	if (s)
 		gtk_entry_set_text (GTK_ENTRY (w), s);
-		druid_data.crash_type = CRASH_DIALOG;
-	}
-	druid_data.app_file = w;
 
-	w = glade_xml_get_widget (xml, "crashed_pid");
-	if (popt_data.pid)
-		s = popt_data.pid;
-	else {
+	package = g_strdup (popt_data.package);
+	if (!package && s)
+		package = get_package_from_appname (s);
+
+	if (!package)
+		package = g_strdup ("general");
+
+	w = glade_xml_get_widget (druid_data.xml, 
+				  "package_entry");
+	gtk_entry_set_text (GTK_ENTRY (w), package);
+	g_free (package);
+
+	w = druid_data.pid =
+		glade_xml_get_widget (druid_data.xml, "crashed_pid");
+	s = popt_data.pid;
+	if (!s) {
 		s = getenv ("GNOME_CRASHED_PID");
 		if (s)
-			g_warning ("$GNOME_CRASHED_PID is deprectated,"
-				   "Please use the --pid command line"
-				   "argument instead.\n");
+			g_warning (_("$GNOME_CRASHED_PID is deprecated.\n "
+				     "Please use the --pid command line"
+				     "argument instead."));
 	}
 
 	if (s) {
 		gtk_entry_set_text (GTK_ENTRY (w), s);
 		druid_data.crash_type = CRASH_DIALOG;
 	}
-	druid_data.pid = w;
 
 	/* core crash page */
-	w = glade_xml_get_widget (xml, "core_file");
+	w = druid_data.core_file = 
+		glade_xml_get_widget (druid_data.xml, "core_file");
 	if (popt_data.core_file) {
 		gtk_entry_set_text (GTK_ENTRY (w), popt_data.core_file);
 		druid_data.crash_type = CRASH_CORE;
 	}
-	druid_data.core_file = w;
 
+	/* package version */
+	if (popt_data.package_ver) {
+		w = glade_xml_get_widget (druid_data.xml, "version_entry");
+		gtk_entry_set_text (GTK_ENTRY (w), popt_data.package_ver);
+	}
 
 	/* init radio buttons */
-	w = glade_xml_get_widget (xml, "dialog_radio");
-	if (druid_data.crash_type == CRASH_DIALOG)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_crash_type),
-			    GINT_TO_POINTER (CRASH_DIALOG));
+	init_toggle ("dialog_radio", druid_data.crash_type, CRASH_DIALOG, 
+		     GTK_SIGNAL_FUNC (update_crash_type));
 
-	w = glade_xml_get_widget (xml, "core_radio");
-	if (druid_data.crash_type == CRASH_CORE)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), 
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_crash_type),
-			    GINT_TO_POINTER (CRASH_CORE));
+	init_toggle ("core_radio", druid_data.crash_type, CRASH_CORE,
+		     GTK_SIGNAL_FUNC (update_crash_type));
 
-	w = glade_xml_get_widget (xml, "no_crash_radio");
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_crash_type),
-			    GINT_TO_POINTER (CRASH_NONE));
+	init_toggle ("no_crash_radio", druid_data.crash_type, CRASH_NONE,
+		     GTK_SIGNAL_FUNC (update_crash_type));
 
-	w = glade_xml_get_widget (xml, "submit_radio");
-	if (druid_data.submit_type == SUBMIT_REPORT)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_submit_type),
-			    GINT_TO_POINTER (SUBMIT_REPORT));
+	init_toggle ("submit_radio", druid_data.submit_type, SUBMIT_REPORT, 
+		     GTK_SIGNAL_FUNC (update_submit_type));
 
-	w = glade_xml_get_widget (xml, "email_radio");
-	if (druid_data.submit_type == SUBMIT_TO_SELF)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_submit_type),
-			    GINT_TO_POINTER (SUBMIT_TO_SELF));
+	init_toggle ("email_radio", druid_data.submit_type, SUBMIT_TO_SELF,
+		     GTK_SIGNAL_FUNC (update_submit_type));
 
-	w = glade_xml_get_widget (xml, "noaction_radio");
-	if (druid_data.submit_type == SUBMIT_NONE)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_submit_type),
-			    GINT_TO_POINTER (SUBMIT_NONE));
-	
-	w = glade_xml_get_widget (xml, "file_radio");
-	if (druid_data.submit_type == SUBMIT_FILE)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-					      TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled",
-			    GTK_SIGNAL_FUNC (update_submit_type),
-			    GINT_TO_POINTER (SUBMIT_FILE));	
+	init_toggle ("noaction_radio", druid_data.submit_type, SUBMIT_NONE,
+		     GTK_SIGNAL_FUNC (update_submit_type));
+
+	init_toggle ("file_radio", druid_data.submit_type, SUBMIT_FILE,
+		     GTK_SIGNAL_FUNC (update_submit_type));
+
+	/* text areas */
+	w = glade_xml_get_widget (druid_data.xml, "desc_area");
+	gtk_text_set_line_wrap (GTK_TEXT (w), FALSE);
+
+	w = glade_xml_get_widget (druid_data.xml, "repeat_area");
+	gtk_text_set_line_wrap (GTK_TEXT (w), FALSE);
 
 	/* system config page */
-	druid_data.version_edit = glade_xml_get_widget (xml, "version_edit");
-	druid_data.version_label = glade_xml_get_widget (xml, "version_label");
+	druid_data.version_edit =
+		glade_xml_get_widget (druid_data.xml, "version_edit");
+	druid_data.version_label = 
+		glade_xml_get_widget (druid_data.xml, "version_label");
 	druid_data.version_list = w =
-		glade_xml_get_widget (xml, "version_list");
+		glade_xml_get_widget (druid_data.xml, "version_list");
 	for (data = list_data; data->label; data++) {
 		for (i = 0; data->cmds[i] && !row[1]; i++)
 			row[1] = get_data_from_command (data->cmds[i]);
@@ -830,27 +889,48 @@ init_ui (GladeXML *xml)
 	}
 
 	/* less page */
-	druid_data.gdb_text = glade_xml_get_widget (xml, "gdb_text");
+	druid_data.stop_button = w = 
+		glade_xml_get_widget (druid_data.xml, "stop_button");
+	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+			    GTK_SIGNAL_FUNC (on_stop_button_clicked),
+			    NULL);
+	gtk_widget_set_sensitive (GTK_WIDGET (w), FALSE);
 
-	druid_data.nature = glade_xml_get_widget (xml, "nature_page");
-	druid_data.attach = glade_xml_get_widget (xml, "attach_page");
-	druid_data.core = glade_xml_get_widget (xml, "core_page");
-	druid_data.less = glade_xml_get_widget (xml, "less_page");
-	druid_data.misc = glade_xml_get_widget (xml, "misc_page");
+	druid_data.refresh_button = w =
+		glade_xml_get_widget (druid_data.xml, "refresh_button");
+	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+			    GTK_SIGNAL_FUNC (on_refresh_button_clicked),
+			    NULL);
+	
+	/* other stuff */
+	druid_data.gdb_text = glade_xml_get_widget (druid_data.xml, 
+						    "gdb_text");
+	druid_data.nature   = glade_xml_get_widget (druid_data.xml,
+						    "nature_page");
+	druid_data.attach   = glade_xml_get_widget (druid_data.xml,
+						    "attach_page");
+	druid_data.core   = glade_xml_get_widget (druid_data.xml,
+						  "core_page");
+	druid_data.less   = glade_xml_get_widget (druid_data.xml, 
+						  "less_page");
+	druid_data.action = glade_xml_get_widget (druid_data.xml, 
+						  "action_page");
 }
 
 int
 main (int argc, char *argv[])
 {
-	GtkWidget *window;
-	gchar *xml_path;
+	GtkWidget *w;
+	gchar *xml_path, *s;
 
 	memset (&druid_data, 0, sizeof (druid_data));
+	memset (&popt_data,  0, sizeof (popt_data));
 
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, options, 0, NULL);
+	gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, 
+				    options, 0, NULL);
 	glade_gnome_init ();
 
 
@@ -868,30 +948,28 @@ main (int argc, char *argv[])
 
 
 	xml_path = BUDDY_DATADIR "/bug-buddy.glade";
-	if (!xml_path) {
-		GtkWidget *d;
-		gchar *s = g_strdup_printf (_("Could not find our glade file '%s'\n"
-					      "Please make sure bug-buddy was "
-					      "installed correctly."), xml_path);
-		d = gnome_error_dialog (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+	if (!g_file_exists (xml_path)) {
+		s = g_strdup_printf (_("Could not find our glade file '%s'\n"
+				       "Please make sure bug-buddy was "
+				       "installed correctly."), xml_path);
+		w = gnome_error_dialog (s);
+		gnome_dialog_run_and_close (GNOME_DIALOG (w));
 		return 0;
 	}
 	druid_data.xml = glade_xml_new (xml_path , "druid_window");
 
 	if (!druid_data.xml) {
-		GtkWidget *d;
-		gchar *s;
-		s = g_strdup_printf (_("XML file '%s' could not be loaded"), xml_path);
-		d = gnome_error_dialog (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		s = g_strdup_printf (_("XML file '%s' could not be loaded"), 
+				     xml_path);
+		w = gnome_error_dialog (s);
+		gnome_dialog_run_and_close (GNOME_DIALOG (w));
 		return 0;
 	}
 
 	init_ui (druid_data.xml);
 
-	window = glade_xml_get_widget (druid_data.xml, "druid_window");
-	gtk_widget_show (window);
+	w = glade_xml_get_widget (druid_data.xml, "druid_window");
+	gtk_widget_show (w);
 
 	gtk_main ();
 
