@@ -1,6 +1,6 @@
 /* bug-buddy bug submitting program
  *
- * Copyright (C) The Free Software Foundation
+ * Copyright (C) Jacob Berkman
  *
  * Author:  Jacob Berkman  <jberkman@andrew.cmu.edu>
  *
@@ -23,6 +23,7 @@
 #define __BUG_BUDDY_H__
 
 #include <glade/glade.h>
+#include <sys/types.h>
 
 #define SUBMIT_ADDRESS "submit@bugs.gnome.org";
 #define COMMAND_SIZE 5
@@ -49,11 +50,11 @@ typedef enum {
 typedef struct {
 	GtkWidget *the_druid;
 
-	GtkWidget *nature;
+	GtkWidget *action;
 	GtkWidget *attach;
 	GtkWidget *core;
 	GtkWidget *less;
-	GtkWidget *misc;
+	GtkWidget *nature;
 
 	GtkWidget *gdb_text;
 	GtkWidget *gdb_anim;
@@ -66,6 +67,9 @@ typedef struct {
 	GtkWidget *version_list;
 	ListData *selected_data;
 
+	GtkWidget *stop_button;
+	GtkWidget *refresh_button;
+
 	gchar *mail_cmd;
 	CrashType crash_type;
 	SubmitType submit_type;
@@ -73,12 +77,22 @@ typedef struct {
 	int bug_class;
 
 	GladeXML *xml;
+
+	pid_t app_pid;
+
+	pid_t gdb_pid;
+	GIOChannel *ioc;
+	int fd;
+
+	gboolean explicit_dirty;
 } DruidData;
 
 extern DruidData druid_data;
 
 void get_trace_from_core (const gchar *core_file);
 void get_trace_from_pair (const gchar *app, const gchar *extra);
+void stop_gdb (void);
+void start_gdb (void);
 
 #endif /* __bug_buddy_h__ */
 
